@@ -225,6 +225,7 @@ static void show_status(void)
 	if (cman_get_quorum_device(h, &qinfo) == 0 && qinfo.qi_state == 2)
 		printf("Quorum device votes: %d\n", qinfo.qi_votes);
 	printf("Total votes: %d\n", einfo->ei_total_votes);
+	printf("Node votes: %d\n", einfo->ei_node_votes);
 
 	printf("Quorum: %d %s\n", einfo->ei_quorum, quorate?" ":"Activity blocked");
 	printf("Active subsystems: %d\n", cman_get_subsys_count(h));
@@ -971,7 +972,7 @@ static void check_arguments(commandline_t *comline)
 		die("timeout is only appropriate with wait");
 }
 
-int main(int argc, char *argv[])
+int main(int argc, char *argv[], char *envp[])
 {
 	commandline_t comline;
 	int ret;
@@ -991,12 +992,12 @@ int main(int argc, char *argv[])
 			alarm(comline.timeout);
 		}
 
-		join(&comline);
+		join(&comline, envp);
 		if (comline.wait_opt || comline.wait_quorate_opt) {
 			do {
 				ret = cluster_wait(&comline);
 				if (ret == ENOTCONN)
-					join(&comline);
+					join(&comline, envp);
 
 			} while (ret == ENOTCONN);
 		}
