@@ -56,9 +56,9 @@ static LIST_INIT(cluster_members_list);
 static struct cluster_node *us;
 static int quorum;
 extern int two_node;
-       unsigned int quorumdev_poll=10000;
-       unsigned int shutdown_timeout=5000;
-       unsigned int ccsd_poll_interval=1000;
+       unsigned int quorumdev_poll=DEFAULT_QUORUMDEV_POLL;
+       unsigned int shutdown_timeout=DEFAULT_SHUTDOWN_TIMEOUT;
+       unsigned int ccsd_poll_interval=DEFAULT_CCSD_POLL;
 static int cluster_is_quorate;
        char cluster_name[MAX_CLUSTER_NAME_LEN+1];
 static char nodename[MAX_CLUSTER_MEMBER_NAME_LEN+1];
@@ -1778,8 +1778,7 @@ static void do_process_transition(int nodeid, char *data, int len)
 
 	/* Newer nodes 6.1.0 onwards, set the DIRTY flag if they have state. If the new node has been down
 	   and has state then we mark it disallowed because we cannot merge stateful nodes */
-	if (msg->flags & NODE_FLAGS_DIRTY && (node->flags & NODE_FLAGS_BEENDOWN ||
-	    (us->flags & NODE_FLAGS_DIRTY && nodeid != us->node_id))) {
+	if (msg->flags & NODE_FLAGS_DIRTY && (node->flags & NODE_FLAGS_BEENDOWN)) {
 		/* Don't duplicate messages */
 		if (node->state != NODESTATE_AISONLY) {
 			if (cluster_is_quorate) {
