@@ -1,4 +1,3 @@
-#include <syslog.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
@@ -11,6 +10,7 @@
 
 #include <openais/service/logsys.h>
 #include "list.h"
+#include "cman.h"
 #include "cnxman-socket.h"
 #include "cnxman-private.h"
 #include "logging.h"
