@@ -31,19 +31,57 @@ struct block_count {
 };
 
 static int leaf(struct gfs2_inode *ip, uint64_t block,
-				struct gfs2_buffer_head *bh, void *private)
+		struct gfs2_buffer_head *bh, void *private);
+static int check_metalist(struct gfs2_inode *ip, uint64_t block,
+			  struct gfs2_buffer_head **bh, void *private);
+static int check_data(struct gfs2_inode *ip, uint64_t block, void *private);
+static int check_eattr_indir(struct gfs2_inode *ip, uint64_t indirect,
+			     uint64_t parent, struct gfs2_buffer_head **bh,
+			     enum update_flags *want_updated, void *private);
+static int check_eattr_leaf(struct gfs2_inode *ip, uint64_t block,
+			    uint64_t parent, struct gfs2_buffer_head **bh,
+			    enum update_flags *want_updated, void *private);
+static int check_eattr_entries(struct gfs2_inode *ip,
+			       struct gfs2_buffer_head *leaf_bh,
+			       struct gfs2_ea_header *ea_hdr,
+			       struct gfs2_ea_header *ea_hdr_prev,
+			       void *private);
+static int check_extended_leaf_eattr(struct gfs2_inode *ip, uint64_t *data_ptr,
+				     struct gfs2_buffer_head *leaf_bh,
+				     struct gfs2_ea_header *ea_hdr,
+				     struct gfs2_ea_header *ea_hdr_prev,
+				     enum update_flags *want_updated,
+				     void *private);
+static int finish_eattr_indir(struct gfs2_inode *ip, int indir_ok,
+			      enum update_flags *want_updated, void *private);
+
+struct metawalk_fxns pass1_fxns = {
+	.private = NULL,
+	.check_leaf = leaf,
+	.check_metalist = check_metalist,
+	.check_data = check_data,
+	.check_eattr_indir = check_eattr_indir,
+	.check_eattr_leaf = check_eattr_leaf,
+	.check_dentry = NULL,
+	.check_eattr_entry = check_eattr_entries,
+	.check_eattr_extentry = check_extended_leaf_eattr,
+	.finish_eattr_indir = finish_eattr_indir,
+};
+
+static int leaf(struct gfs2_inode *ip, uint64_t block,
+		struct gfs2_buffer_head *bh, void *private)
 {
 	struct block_count *bc = (struct block_count *) private;
 
 	log_debug("\tLeaf block at %15" PRIu64 " (0x%" PRIx64 ")\n",
 			  block, block);
-	gfs2_block_set(bl, block, gfs2_leaf_blk);
+	gfs2_block_set(ip->i_sbd, bl, block, gfs2_leaf_blk);
 	bc->indir_count++;
 	return 0;
 }
 
 static int check_metalist(struct gfs2_inode *ip, uint64_t block,
-						  struct gfs2_buffer_head **bh, void *private)
+			  struct gfs2_buffer_head **bh, void *private)
 {
 	struct gfs2_block_query q = {0};
 	int found_dup = 0;
@@ -53,19 +91,20 @@ static int check_metalist(struct gfs2_inode *ip, uint64_t block,
 	*bh = NULL;
 
 	if (gfs2_check_range(ip->i_sbd, block)){ /* blk outside of FS */
-		gfs2_block_set(bl, ip->i_di.di_num.no_addr, gfs2_bad_block);
+		gfs2_block_set(ip->i_sbd, bl, ip->i_di.di_num.no_addr,
+			       gfs2_bad_block);
 		log_debug("Bad indirect block pointer (out of range).\n");
 
 		return 1;
 	}
-	if(gfs2_block_check(bl, block, &q)) {
+	if(gfs2_block_check(ip->i_sbd, bl, block, &q)) {
 		stack;
 		return -1;
 	}
 	if(q.block_type != gfs2_block_free) {
 		log_debug("Found duplicate block in indirect block -"
 				  " was marked %d\n", q.block_type);
-		gfs2_block_mark(bl, block, gfs2_dup_block);
+		gfs2_block_mark(ip->i_sbd, bl, block, gfs2_dup_block);
 		found_dup = 1;
 	}
 	nbh = bread(ip->i_sbd, block);
@@ -74,7 +113,7 @@ static int check_metalist(struct gfs2_inode *ip, uint64_t block,
 		log_debug("Bad indirect block pointer "
 				  "(points to something that is not an indirect block).\n");
 		if(!found_dup) {
-			gfs2_block_set(bl, block, gfs2_meta_inval);
+			gfs2_block_set(ip->i_sbd, bl, block, gfs2_meta_inval);
 			brelse(nbh, not_updated);
 			return 1;
 		}
@@ -83,7 +122,7 @@ static int check_metalist(struct gfs2_inode *ip, uint64_t block,
 
 	log_debug("Setting %" PRIu64 " (0x%" PRIx64 ") to indirect block.\n",
 			  block, block);
-	gfs2_block_set(bl, block, gfs2_indir_blk);
+	gfs2_block_set(ip->i_sbd, bl, block, gfs2_indir_blk);
 	bc->indir_count++;
 
 	return 0;
@@ -98,30 +137,77 @@ static int check_data(struct gfs2_inode *ip, uint64_t block, void *private)
 		log_err( "Bad data block pointer (out of range)\n");
 		/* Mark the owner of this block with the bad_block
 		 * designator so we know to check it for out of range blocks later */
-		gfs2_block_set(bl, ip->i_di.di_num.no_addr, gfs2_bad_block);
+		gfs2_block_set(ip->i_sbd, bl, ip->i_di.di_num.no_addr,
+			       gfs2_bad_block);
 		return 1;
 	}
-	if(gfs2_block_check(bl, block, &q)) {
+	if(gfs2_block_check(ip->i_sbd, bl, block, &q)) {
 		stack;
 		return -1;
 	}
 	if(q.block_type != gfs2_block_free) {
 		log_debug("Found duplicate block at %" PRIu64 " (0x%"PRIx64 ")\n",
 				  block, block);
-		gfs2_block_mark(bl, block, gfs2_dup_block);
+		gfs2_block_mark(ip->i_sbd, bl, block, gfs2_dup_block);
 		bc->data_count++;
 		return 1;
 	}
 	log_debug("Setting %" PRIu64 " (0x%" PRIx64 ") to data block\n", block,
 			  block);
-	gfs2_block_set(bl, block, gfs2_block_used);
+	gfs2_block_set(ip->i_sbd, bl, block, gfs2_block_used);
 	bc->data_count++;
 	return 0;
 }
 
+/* clear_eas - clear the extended attributes for an inode
+ *
+ * @ip       - in core inode pointer
+ * @bc       - pointer to a block count structure
+ * block     - the block that had the problem
+ * duplicate - if this is a duplicate block, don't set it "free"
+ * emsg      - what to tell the user about the eas being checked
+ * Returns: 1 if the EA is fixed, else 0 if it was not fixed.
+ */
+static int clear_eas(struct gfs2_inode *ip, struct block_count *bc,
+		     uint64_t block, int duplicate,
+		     enum update_flags *want_updated, const char *emsg)
+{
+	struct gfs2_sbd *sdp = ip->i_sbd;
+
+	*want_updated = not_updated;
+	log_err("Inode #%" PRIu64 " (0x%" PRIx64 "): %s",
+		ip->i_di.di_num.no_addr, ip->i_di.di_num.no_addr, emsg);
+	if (block)
+		log_err(" at block #%" PRIu64 " (0x%" PRIx64 ")",
+			block, block);
+	log_err(".\n");
+	if (query(&opts, "Clear the bad EA? (y/n) ")) {
+		if (block == 0)
+			block = ip->i_di.di_eattr;
+		gfs2_block_clear(sdp, bl, block, gfs2_eattr_block);
+		if (!duplicate) {
+			gfs2_block_clear(sdp, bl, block, gfs2_indir_blk);
+			gfs2_block_set(sdp, bl, block, gfs2_block_free);
+			gfs2_set_bitmap(sdp, block, GFS2_BLKST_FREE);
+		}
+		ip->i_di.di_flags &= ~GFS2_DIF_EA_INDIRECT;
+		if (block == ip->i_di.di_eattr)
+			ip->i_di.di_eattr = 0;
+		bc->ea_count = 0;
+		ip->i_di.di_blocks = 1 + bc->indir_count + bc->data_count;
+		gfs2_dinode_out(&ip->i_di, ip->i_bh->b_data);
+		*want_updated = updated;
+		return 1;
+	} else {
+		log_err("The bad EA was not fixed.\n");
+		bc->ea_count++;
+		return 0;
+	}
+}
+
 static int check_eattr_indir(struct gfs2_inode *ip, uint64_t indirect,
-							 uint64_t parent, struct gfs2_buffer_head **bh,
-							 void *private)
+			     uint64_t parent, struct gfs2_buffer_head **bh,
+			     enum update_flags *want_updated, void *private)
 {
 	struct gfs2_sbd *sdp = ip->i_sbd;
 	int ret = 0;
@@ -133,48 +219,67 @@ static int check_eattr_indir(struct gfs2_inode *ip, uint64_t indirect,
 	if(gfs2_check_range(sdp, indirect)) {
 		/*log_warn("EA indirect block #%"PRIu64" is out of range.\n",
 			indirect);
-			gfs2_block_set(bl, parent, bad_block);*/
+			gfs2_block_set(sdp, bl, parent, bad_block);*/
 		/* Doesn't help to mark this here - this gets checked
 		 * in pass1c */
-		ret = 1;
+		return 1;
 	}
-	else if(gfs2_block_check(bl, indirect, &q)) {
+	if(gfs2_block_check(sdp, bl, indirect, &q)) {
 		stack;
-		ret = -1;
+		return -1;
 	}
-	else if(q.block_type != gfs2_block_free) {
-		log_debug("Duplicate block found at #%" PRIu64 " (0x%" PRIx64 ").\n",
-				  indirect, indirect);
-		gfs2_block_set(bl, indirect, gfs2_dup_block);
+
+	/* Special duplicate processing:  If we have an EA block,
+	   check if it really is an EA.  If it is, let duplicate
+	   handling sort it out.  If it isn't, clear it but don't
+	   count it as a duplicate. */
+	*bh = bread(sdp, indirect);
+	if(gfs2_check_meta(*bh, GFS2_METATYPE_IN)) {
+		if(q.block_type != gfs2_block_free) { /* Duplicate? */
+			if (!clear_eas(ip, bc, indirect, 1, want_updated,
+				       "Bad indirect EA duplicate found"))
+				gfs2_block_set(sdp, bl, indirect,
+					       gfs2_dup_block);
+			return 1;
+		}
+		clear_eas(ip, bc, indirect, 0, want_updated,
+			  "EA indirect block has incorrect type");
+		return 1;
+	}
+	if(q.block_type != gfs2_block_free) { /* Duplicate? */
+		log_err("Inode #%" PRIu64 " (0x%" PRIx64
+			"): Duplicate EA indirect block found at #%" PRIu64
+			" (0x%" PRIx64 ").\n",
+			ip->i_di.di_num.no_addr, ip->i_di.di_num.no_addr,
+			indirect, indirect);
+		gfs2_block_set(sdp, bl, indirect, gfs2_dup_block);
 		bc->ea_count++;
 		ret = 1;
+	} else {
+		log_debug("Setting #%" PRIu64 " (0x%" PRIx64
+			  ") to indirect EA block\n", indirect, indirect);
+		gfs2_block_set(sdp, bl, indirect, gfs2_indir_blk);
+		bc->ea_count++;
 	}
-	else {
-		log_debug("Marking inode %" PRIu64 " (0x%"
+	return ret;
+}
+
+static int finish_eattr_indir(struct gfs2_inode *ip, int indir_ok,
+			      enum update_flags *want_updated, void *private)
+{
+	if (indir_ok) {
+		log_debug("Marking inode #%" PRIu64 " (0x%"
 			  PRIx64 ") with eattr block\n",
 			  ip->i_di.di_num.no_addr, ip->i_di.di_num.no_addr);
 		/* Mark the inode as having an eattr in the block map
 		   so pass1c can check it. */
-		gfs2_block_mark(bl, ip->i_di.di_num.no_addr, gfs2_eattr_block);
-
-		*bh = bread(sdp, indirect);
-		if(gfs2_check_meta(*bh, GFS2_METATYPE_IN)) {
-			log_warn("EA indirect block %" PRIu64 " (0x%" PRIx64
-				 ") has incorrect type.\n",
-				 indirect, indirect);
-			gfs2_block_set(bl, indirect, gfs2_meta_inval);
-			ret = 1;
-			brelse(*bh, not_updated);
-		}
-		else {
-			/* FIXME: do i need to differentiate this as an ea_indir? */
-			log_debug("Setting %" PRIu64 " (0x%" PRIx64
-				  ") to indirect block\n", indirect, indirect);
-			gfs2_block_set(bl, indirect, gfs2_indir_blk);
-			bc->ea_count++;
-		}
+		gfs2_block_mark(ip->i_sbd, bl, ip->i_di.di_num.no_addr,
+				gfs2_eattr_block);
+		return 0;
 	}
-	return ret;
+	clear_eas(ip, (struct block_count *)private, 0, 0, want_updated,
+		  "has unrecoverable indirect EA errors");
+	return 0;
 }
 
 /**
@@ -192,6 +297,7 @@ static int check_extended_leaf_eattr(struct gfs2_inode *ip, uint64_t *data_ptr,
 				     struct gfs2_buffer_head *leaf_bh,
 				     struct gfs2_ea_header *ea_hdr,
 				     struct gfs2_ea_header *ea_hdr_prev,
+				     enum update_flags *want_updated,
 				     void *private)
 {
 	struct gfs2_buffer_head *el_buf;
@@ -199,45 +305,60 @@ static int check_extended_leaf_eattr(struct gfs2_inode *ip, uint64_t *data_ptr,
 	struct gfs2_block_query q;
 	uint64_t el_blk = be64_to_cpu(*data_ptr);
 	struct block_count *bc = (struct block_count *) private;
+	int ret = 0;
 
 	if(gfs2_check_range(sdp, el_blk)){
-		log_err("EA extended leaf block #%" PRIu64 " (0x%" PRIx64
-				") is out of range.\n",	el_blk, el_blk);
-		gfs2_block_set(bl, ip->i_di.di_eattr, gfs2_bad_block);
+		log_err("Inode #%" PRIu64 " (0x%" PRIx64 "): EA extended "
+			"leaf block #%" PRIu64 " (0x%" PRIx64
+			") is out of range.\n",
+			ip->i_di.di_num.no_addr, ip->i_di.di_num.no_addr,
+			el_blk, el_blk);
+		gfs2_block_set(sdp, bl, ip->i_di.di_eattr, gfs2_bad_block);
 		return 1;
 	}
 
-	if(gfs2_block_check(bl, el_blk, &q)) {
+	if(gfs2_block_check(sdp, bl, el_blk, &q)) {
 		stack;
 		return -1;
 	}
-	if(q.block_type != gfs2_block_free) {
-		gfs2_block_set(bl, el_blk, gfs2_dup_block);
-		bc->ea_count++;
-		return 1;
-	}
-
 	el_buf = bread(sdp, el_blk);
 
+	/* Special duplicate processing:  If we have an EA block,
+	   check if it really is an EA.  If it is, let duplicate
+	   handling sort it out.  If it isn't, clear it but don't
+	   count it as a duplicate. */
 	if(gfs2_check_meta(el_buf, GFS2_METATYPE_ED)) {
-		log_err("EA extended leaf block %" PRIu64 " (0x%" PRIx64
-				") has incorrect type.\n", el_blk, el_blk);
-		brelse(el_buf, not_updated);
-		gfs2_block_set(bl, el_blk, gfs2_meta_inval);
-		return 1;
+		if(q.block_type != gfs2_block_free) /* Duplicate? */
+			clear_eas(ip, bc, el_blk, 1, want_updated,
+				  "has bad extended EA duplicate");
+		else
+			clear_eas(ip, bc, el_blk, 0, want_updated,
+				  "EA extended leaf block has incorrect type");
+		ret = 1;
+	} else { /* If this looks like an EA */
+		if(q.block_type != gfs2_block_free) { /* Duplicate? */
+			log_debug("Duplicate block found at #%" PRIu64
+				  " (0x%" PRIx64 ").\n",
+				  el_blk, el_blk);
+			gfs2_block_set(sdp, bl, el_blk, gfs2_dup_block);
+			bc->ea_count++;
+			ret = 1;
+		} else {
+			log_debug("Setting block #%" PRIu64
+				  " (0x%" PRIx64 ") to eattr block\n",
+				  el_blk, el_blk);
+			gfs2_block_set(sdp, bl, el_blk, gfs2_meta_eattr);
+			bc->ea_count++;
+		}
 	}
 
-	log_debug("Setting %" PRIu64 " (0x%" PRIx64 ") to eattr block\n",
-			  el_blk, el_blk);
-	gfs2_block_set(bl, el_blk, gfs2_meta_eattr);
-	bc->ea_count++;
 	brelse(el_buf, not_updated);
-	return 0;
+	return ret;
 }
 
 static int check_eattr_leaf(struct gfs2_inode *ip, uint64_t block,
 			    uint64_t parent, struct gfs2_buffer_head **bh,
-			    void *private)
+			    enum update_flags *want_updated, void *private)
 {
 	struct gfs2_sbd *sdp = ip->i_sbd;
 	struct gfs2_buffer_head *leaf_bh = NULL;
@@ -250,47 +371,57 @@ static int check_eattr_leaf(struct gfs2_inode *ip, uint64_t block,
 	if (parent != ip->i_di.di_num.no_addr) { /* if parent isn't the inode */
 		log_debug("Setting %" PRIu64 " (0x%" PRIx64 ") to eattr block\n",
 				  parent, parent);
-		gfs2_block_set(bl, parent, gfs2_eattr_block);
+		gfs2_block_set(sdp, bl, parent, gfs2_eattr_block);
 	}
 	if(gfs2_check_range(sdp, block)){
-		log_warn("EA leaf block #%" PRIu64 " (0x%" PRIx64 ") in inode %" PRIu64
-				 " (0x%" PRIx64 ") is out of range.\n",
-				 ip->i_di.di_num.no_addr, ip->i_di.di_num.no_addr,
-				 block, block);
-		gfs2_block_set(bl, ip->i_di.di_eattr, gfs2_bad_block);
+		log_warn("Inode #%" PRIu64 " (0x%" PRIx64 "): EA leaf block "
+			 "#%" PRIu64 " (0x%" PRIx64 ") is out of range.\n",
+			 ip->i_di.di_num.no_addr, ip->i_di.di_num.no_addr,
+			 block, block);
+		gfs2_block_set(sdp, bl, ip->i_di.di_eattr, gfs2_bad_block);
 		ret = 1;
 	}
-	else if(gfs2_block_check(bl, block, &q)) {
+	else if(gfs2_block_check(sdp, bl, block, &q)) {
 		stack;
 		return -1;
 	}
-	else if(q.block_type != gfs2_block_free) {
-		log_debug("Duplicate block found at #%" PRIu64 " (0x%" PRIx64 ").\n",
-				  block, block);
-		gfs2_block_set(bl, block, gfs2_dup_block);
-		bc->ea_count++;
-		ret = 1;
-	}
 	else {
+		/* Special duplicate processing:  If we have an EA block,
+		   check if it really is an EA.  If it is, let duplicate
+		   handling sort it out.  If it isn't, clear it but don't
+		   count it as a duplicate. */
 		leaf_bh = bread(sdp, block);
 		if(gfs2_check_meta(leaf_bh, GFS2_METATYPE_EA)) {
-			log_warn("EA leaf block %"PRIu64" (0x%" 
-				 PRIx64") for inode %"PRIu64" (0x%"
-				 PRIx64") has incorrect type.\n",
-				 block, block, ip->i_di.di_num.no_addr,
-				 ip->i_di.di_num.no_addr);
-			gfs2_block_set(bl, block, gfs2_meta_inval);
-			ret = -1;
+			if(q.block_type != gfs2_block_free) { /* Duplicate? */
+				clear_eas(ip, bc, block, 1, want_updated,
+					  "Bad EA duplicate found");
+			} else {
+				clear_eas(ip, bc, block, 0, want_updated,
+					  "EA leaf block has incorrect type");
+			}
+			ret = 1;
 			brelse(leaf_bh, not_updated);
-		}
-		else {
-			log_debug("Setting block %" PRIu64 " (0x%" PRIx64
-					  ") to eattr block\n", block, block);
-			gfs2_block_set(bl, block, gfs2_meta_eattr);
-			bc->ea_count++;
+		} else { /* If this looks like an EA */
+			if(q.block_type != gfs2_block_free) { /* Duplicate? */
+				log_debug("Duplicate block found at #%" PRIu64
+					  " (0x%" PRIx64 ").\n",
+					  block, block);
+				gfs2_block_set(sdp, bl, block, gfs2_dup_block);
+				bc->ea_count++;
+				ret = 1;
+				brelse(leaf_bh, not_updated);
+			} else {
+				log_debug("Setting block #%" PRIu64
+					  " (0x%" PRIx64 ") to eattr block\n",
+					  block, block);
+				gfs2_block_set(sdp, bl, block,
+					       gfs2_meta_eattr);
+				bc->ea_count++;
+			}
 		}
 	}
-	*bh = leaf_bh;
+	if (!ret)
+		*bh = leaf_bh;
 
 	return ret;
 }
@@ -336,18 +467,6 @@ static int check_eattr_entries(struct gfs2_inode *ip,
 	return 0;
 }
 
-struct metawalk_fxns pass1_fxns = {
-	.private = NULL,
-	.check_leaf = leaf,
-	.check_metalist = check_metalist,
-	.check_data = check_data,
-	.check_eattr_indir = check_eattr_indir,
-	.check_eattr_leaf = check_eattr_leaf,
-	.check_dentry = NULL,
-	.check_eattr_entry = check_eattr_entries,
-	.check_eattr_extentry = check_extended_leaf_eattr,
-};
-
 int clear_metalist(struct gfs2_inode *ip, uint64_t block,
 		   struct gfs2_buffer_head **bh, void *private)
 {
@@ -355,12 +474,12 @@ int clear_metalist(struct gfs2_inode *ip, uint64_t block,
 
 	*bh = NULL;
 
-	if(gfs2_block_check(bl, block, &q)) {
+	if(gfs2_block_check(ip->i_sbd, bl, block, &q)) {
 		stack;
 		return -1;
 	}
 	if(!q.dup_block) {
-		gfs2_block_set(bl, block, gfs2_block_free);
+		gfs2_block_set(ip->i_sbd, bl, block, gfs2_block_free);
 		return 0;
 	}
 	return 0;
@@ -370,12 +489,12 @@ int clear_data(struct gfs2_inode *ip, uint64_t block, void *private)
 {
 	struct gfs2_block_query q = {0};
 
-	if(gfs2_block_check(bl, block, &q)) {
+	if(gfs2_block_check(ip->i_sbd, bl, block, &q)) {
 		stack;
 		return -1;
 	}
 	if(!q.dup_block) {
-		gfs2_block_set(bl, block, gfs2_block_free);
+		gfs2_block_set(ip->i_sbd, bl, block, gfs2_block_free);
 		return 0;
 	}
 	return 0;
@@ -386,16 +505,16 @@ int clear_leaf(struct gfs2_inode *ip, uint64_t block,
 	       struct gfs2_buffer_head *bh, void *private)
 {
 	struct gfs2_block_query q = {0};
-	log_crit("Clearing leaf %" PRIu64 " (0x%" PRIx64 ")\n", block, block);
+	log_crit("Clearing leaf #%" PRIu64 " (0x%" PRIx64 ")\n", block, block);
 
-	if(gfs2_block_check(bl, block, &q)) {
+	if(gfs2_block_check(ip->i_sbd, bl, block, &q)) {
 		stack;
 		return -1;
 	}
 	if(!q.dup_block) {
-		log_crit("Setting leaf %" PRIu64 " (0x%" PRIx64 ") invalid\n",
+		log_crit("Setting leaf #%" PRIu64 " (0x%" PRIx64 ") invalid\n",
 				 block, block);
-		if(gfs2_block_set(bl, block, gfs2_block_free)) {
+		if(gfs2_block_set(ip->i_sbd, bl, block, gfs2_block_free)) {
 			stack;
 			return -1;
 		}
@@ -416,7 +535,7 @@ int add_to_dir_list(struct gfs2_sbd *sbp, uint64_t block)
 	 * matter too much */
 	find_di(sbp, block, &di);
 	if(di) {
-		log_err("Attempting to add directory block %" PRIu64
+		log_err("Attempting to add directory block #%" PRIu64
 				" (0x%" PRIx64 ") which is already in list\n", block, block);
 		return -1;
 	}
@@ -456,25 +575,25 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 				"): Bad inode address found: %"	PRIu64 " (0x%" PRIx64 ")\n",
 				block, block, ip->i_di.di_num.no_addr,
 				ip->i_di.di_num.no_addr);
-		if(query(&opts, "Fix address in inode at block %"
+		if(query(&opts, "Fix address in inode at block #%"
 				 PRIu64 " (0x%" PRIx64 ")? (y/n) ", block, block)) {
 			ip->i_di.di_num.no_addr = ip->i_di.di_num.no_formal_ino = block;
 			gfs2_dinode_out(&ip->i_di, ip->i_bh->b_data);
 			f = updated;
 		} else
-			log_err("Address in inode at block %" PRIu64
+			log_err("Address in inode at block #%" PRIu64
 				 " (0x%" PRIx64 ") not fixed\n", block, block);
 	}
 
-	if(gfs2_block_check(bl, block, &q)) {
+	if(gfs2_block_check(sdp, bl, block, &q)) {
 		stack;
 		fsck_inode_put(ip, f);
 		return -1;
 	}
 	if(q.block_type != gfs2_block_free) {
-		log_debug("Found duplicate block at %" PRIu64 " (0x%" PRIx64 ")\n",
+		log_debug("Found duplicate block at #%" PRIu64 " (0x%" PRIx64 ")\n",
 				  block, block);
-		if(gfs2_block_mark(bl, block, gfs2_dup_block)) {
+		if(gfs2_block_mark(sdp, bl, block, gfs2_dup_block)) {
 			stack;
 			fsck_inode_put(ip, f);
 			return -1;
@@ -488,7 +607,7 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 	case S_IFDIR:
 		log_debug("Setting %" PRIu64 " (0x%" PRIx64 ") to directory inode.\n",
 				  block, block);
-		if(gfs2_block_set(bl, block, gfs2_inode_dir)) {
+		if(gfs2_block_set(sdp, bl, block, gfs2_inode_dir)) {
 			stack;
 			fsck_inode_put(ip, f);
 			return -1;
@@ -502,7 +621,7 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 	case S_IFREG:
 		log_debug("Setting %" PRIu64 " (0x%" PRIx64 ") to file inode.\n",
 				  block, block);
-		if(gfs2_block_set(bl, block, gfs2_inode_file)) {
+		if(gfs2_block_set(sdp, bl, block, gfs2_inode_file)) {
 			stack;
 			fsck_inode_put(ip, f);
 			return -1;
@@ -511,7 +630,7 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 	case S_IFLNK:
 		log_debug("Setting %" PRIu64 " (0x%" PRIx64 ") to symlink inode.\n",
 				  block, block);
-		if(gfs2_block_set(bl, block, gfs2_inode_lnk)) {
+		if(gfs2_block_set(sdp, bl, block, gfs2_inode_lnk)) {
 			stack;
 			fsck_inode_put(ip, f);
 			return -1;
@@ -520,7 +639,7 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 	case S_IFBLK:
 		log_debug("Setting %" PRIu64 " (0x%" PRIx64 ") to block dev inode.\n",
 				  block, block);
-		if(gfs2_block_set(bl, block, gfs2_inode_blk)) {
+		if(gfs2_block_set(sdp, bl, block, gfs2_inode_blk)) {
 			stack;
 			fsck_inode_put(ip, f);
 			return -1;
@@ -529,7 +648,7 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 	case S_IFCHR:
 		log_debug("Setting %" PRIu64 " (0x%" PRIx64 ") to char dev inode.\n",
 				  block, block);
-		if(gfs2_block_set(bl, block, gfs2_inode_chr)) {
+		if(gfs2_block_set(sdp, bl, block, gfs2_inode_chr)) {
 			stack;
 			fsck_inode_put(ip, f);
 			return -1;
@@ -538,7 +657,7 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 	case S_IFIFO:
 		log_debug("Setting %" PRIu64 " (0x%" PRIx64 ") to fifo inode.\n",
 				  block, block);
-		if(gfs2_block_set(bl, block, gfs2_inode_fifo)) {
+		if(gfs2_block_set(sdp, bl, block, gfs2_inode_fifo)) {
 			stack;
 			fsck_inode_put(ip, f);
 			return -1;
@@ -547,7 +666,7 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 	case S_IFSOCK:
 		log_debug("Setting %" PRIu64 " (0x%" PRIx64 ") to socket inode.\n",
 				  block, block);
-		if(gfs2_block_set(bl, block, gfs2_inode_sock)) {
+		if(gfs2_block_set(sdp, bl, block, gfs2_inode_sock)) {
 			stack;
 			fsck_inode_put(ip, f);
 			return -1;
@@ -556,11 +675,12 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 	default:
 		log_debug("Setting %" PRIu64 " (0x%" PRIx64 ") to invalid.\n",
 				  block, block);
-		if(gfs2_block_set(bl, block, gfs2_meta_inval)) {
+		if(gfs2_block_set(sdp, bl, block, gfs2_meta_inval)) {
 			stack;
 			fsck_inode_put(ip, f);
 			return -1;
 		}
+		gfs2_set_bitmap(sdp, block, GFS2_BLKST_FREE);
 		fsck_inode_put(ip, f);
 		return 0;
 	}
@@ -579,11 +699,12 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 				 compute_height(sdp, ip->i_di.di_size));
 			/* once implemented, remove continue statement */
 		log_warn("Marking inode invalid\n");
-		if(gfs2_block_set(bl, block, gfs2_meta_inval)) {
+		if(gfs2_block_set(sdp, bl, block, gfs2_meta_inval)) {
 			stack;
 			fsck_inode_put(ip, f);
 			return -1;
 		}
+		gfs2_set_bitmap(sdp, block, GFS2_BLKST_FREE);
 		fsck_inode_put(ip, f);
 		return 0;
 	}
@@ -598,11 +719,12 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 					 (1 >> (ip->i_di.di_size/sizeof(uint64_t))));
 			/* once implemented, remove continue statement */
 			log_warn("Marking inode invalid\n");
-			if(gfs2_block_set(bl, block, gfs2_meta_inval)) {
+			if(gfs2_block_set(sdp, bl, block, gfs2_meta_inval)) {
 				stack;
 				fsck_inode_put(ip, f);
 				return -1;
 			}
+			gfs2_set_bitmap(sdp, block, GFS2_BLKST_FREE);
 			fsck_inode_put(ip, f);
 			return 0;
 		}
@@ -616,23 +738,22 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 		return 0;
 	}
 	if(error > 0) {
-		log_warn("Marking inode invalid\n");
+		log_warn("Marking inode #%" PRIu64 " (0x%" PRIx64 ") invalid\n",
+			 ip->i_di.di_num.no_addr, ip->i_di.di_num.no_addr);
 		/* FIXME: Must set all leaves invalid as well */
 		check_metatree(ip, &invalidate_metatree);
-		gfs2_block_set(bl, ip->i_di.di_num.no_addr, gfs2_meta_inval);
+		gfs2_block_set(sdp, bl, ip->i_di.di_num.no_addr,
+			       gfs2_meta_inval);
+		gfs2_set_bitmap(sdp, ip->i_di.di_num.no_addr, GFS2_BLKST_FREE);
 		fsck_inode_put(ip, f);
 		return 0;
 	}
 
-	if(check_inode_eattr(ip, &pass1_fxns) < 0){
-		ip->i_di.di_eattr = 0;
-		gfs2_dinode_out(&ip->i_di, ip->i_bh->b_data);
-		f = updated;
-	}
+	check_inode_eattr(ip, &f, &pass1_fxns);
 
 	if (ip->i_di.di_blocks != 
 		(1 + bc.indir_count + bc.data_count + bc.ea_count)) {
-		log_err("Inode %" PRIu64 " (0x%" PRIx64 "): Ondisk block count (%"
+		log_err("Inode #%" PRIu64 " (0x%" PRIx64 "): Ondisk block count (%"
 				PRIu64 ") does not match what fsck found (%" PRIu64 ")\n",
 				ip->i_di.di_num.no_addr, ip->i_di.di_num.no_addr,
 				ip->i_di.di_blocks,
@@ -643,7 +764,7 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 			gfs2_dinode_out(&ip->i_di, ip->i_bh->b_data);
 			f = updated;
 		} else
-			log_err("Bad block count for %" PRIu64 " (0x%" PRIx64
+			log_err("Bad block count for #%" PRIu64 " (0x%" PRIx64
 					") not fixed\n", ip->i_di.di_num.no_addr,
 					ip->i_di.di_num.no_addr);
 	}
@@ -656,16 +777,17 @@ int scan_meta(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 			  uint64_t block)
 {
 	if (gfs2_check_meta(bh, 0)) {
-		log_debug("Found invalid metadata at %" PRIu64 " (0x%" PRIx64 ")\n",
+		log_debug("Found invalid metadata at #%" PRIu64 " (0x%" PRIx64 ")\n",
 				  block, block);
-		if(gfs2_block_set(bl, block, gfs2_meta_inval)) {
+		if(gfs2_block_set(sdp, bl, block, gfs2_meta_inval)) {
 			stack;
 			return -1;
 		}
+		gfs2_set_bitmap(sdp, block, GFS2_BLKST_FREE);
 		return 0;
 	}
 
-	log_debug("Checking metadata block %" PRIu64 " (0x%" PRIx64 ")\n", block,
+	log_debug("Checking metadata block #%" PRIu64 " (0x%" PRIx64 ")\n", block,
 			  block);
 
 	if (!gfs2_check_meta(bh, GFS2_METATYPE_DI)) {
@@ -739,7 +861,8 @@ int pass1(struct gfs2_sbd *sbp)
 		log_debug("RG at %" PRIu64 " (0x%" PRIx64 ") is %u long\n",
 				  rgd->ri.ri_addr, rgd->ri.ri_addr, rgd->ri.ri_length);
 		for (i = 0; i < rgd->ri.ri_length; i++) {
-			if(gfs2_block_set(bl, rgd->ri.ri_addr + i, gfs2_meta_other)){
+			if(gfs2_block_set(sbp, bl, rgd->ri.ri_addr + i,
+					  gfs2_meta_other)){
 				stack;
 				return -1;
 			}
