@@ -22,6 +22,7 @@
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <openais/cpg.h>
+#include <openais/service/logsys.h>
 
 #include "list.h"
 #include "linux_endian.h"
@@ -46,33 +47,60 @@ extern struct list_head gd_groups;
 extern struct list_head gd_levels[MAX_LEVELS];
 extern uint32_t gd_event_nr;
 
+#define GROUP_PENDING		1
+#define GROUP_LIBGROUP          2
+#define GROUP_LIBCPG            3
+
+extern int group_mode;
+
+#define DEFAULT_GROUPD_COMPAT		2
+#define DEFAULT_GROUPD_WAIT		5
+#define DEFAULT_GROUPD_MODE_DELAY	2
+#define DEFAULT_DEBUG_LOGSYS		0
+
+extern int optd_groupd_compat;
+extern int optd_groupd_wait;
+extern int optd_groupd_mode_delay;
+extern int optd_debug_logsys;
+
+extern int cfgd_groupd_compat;
+extern int cfgd_groupd_wait;
+extern int cfgd_groupd_mode_delay;
+extern int cfgd_debug_logsys;
+
 void daemon_dump_save(void);
 
 #define log_debug(fmt, args...) \
 do { \
 	snprintf(daemon_debug_buf, 255, "%ld " fmt "\n", time(NULL), ##args); \
-	if (daemon_debug_opt) fprintf(stderr, "%s", daemon_debug_buf); \
 	daemon_dump_save(); \
+	if (daemon_debug_opt) \
+		fprintf(stderr, "%s", daemon_debug_buf); \
+	if (cfgd_debug_logsys) \
+		log_printf(LOG_DEBUG, "%s", daemon_debug_buf); \
 } while (0)
 
 #define log_group(g, fmt, args...) \
 do { \
 	snprintf(daemon_debug_buf, 255, "%ld %d:%s " fmt "\n", time(NULL), \
 		 (g)->level, (g)->name, ##args); \
-	if (daemon_debug_opt) fprintf(stderr, "%s", daemon_debug_buf); \
 	daemon_dump_save(); \
+	if (daemon_debug_opt) \
+		fprintf(stderr, "%s", daemon_debug_buf); \
+	if (cfgd_debug_logsys) \
+		log_printf(LOG_DEBUG, "%s", daemon_debug_buf); \
 } while (0)
 
 #define log_print(fmt, args...) \
 do { \
 	log_debug(fmt, ##args); \
-	syslog(LOG_ERR, fmt, ##args); \
+	log_printf(LOG_ERR, fmt, ##args); \
 } while (0)
 
 #define log_error(g, fmt, args...) \
 do { \
 	log_group(g, fmt, ##args); \
-	syslog(LOG_ERR, fmt, ##args); \
+	log_printf(LOG_ERR, fmt, ##args); \
 } while (0)
 
 #define ASSERT(x) \
@@ -172,9 +200,10 @@ struct app {
 #define MSG_APP_RECOVER        3
 #define MSG_APP_INTERNAL       4
 #define MSG_GLOBAL_ID          5
+#define MSG_GROUP_VERSION      6
 
 #define MSG_VER_MAJOR          1
-#define MSG_VER_MINOR          0
+#define MSG_VER_MINOR          1
 #define MSG_VER_PATCH          0
 
 struct msg {
@@ -247,6 +276,9 @@ int is_our_join(event_t *ev);
 void purge_node_messages(group_t *g, int nodeid);
 
 /* main.c */
+void read_ccs_name(char *path, char *name);
+void read_ccs_yesno(char *path, int *yes, int *no);
+void read_ccs_int(char *path, int *config_val);
 void app_stop(app_t *a);
 void app_setid(app_t *a);
 void app_start(app_t *a);
@@ -272,6 +304,7 @@ int send_message(group_t *g, void *buf, int len);
 int send_message_groupd(group_t *g, void *buf, int len, int type);
 void copy_groupd_data(group_data_t *data);
 int in_groupd_cpg(int nodeid);
+void group_mode_check_timeout(void);
 
 /* joinleave.c */
 void remove_group(group_t *g);
@@ -279,6 +312,14 @@ int do_join(char *name, int level, int ci);
 int do_leave(char *name, int level);
 node_t *new_node(int nodeid);
 group_t *find_group_level(char *name, int level);
+int create_group(char *name, int level, group_t **g_out);
+app_t *create_app(group_t *g);
+
+/* logging.c */
+
+void init_logging(void);
+void setup_logging();
+void close_logging(void);
 
 #endif				/* __GD_INTERNAL_DOT_H__ */
 
