@@ -3,6 +3,7 @@
 
 #include "gd_internal.h"
 #include "ccs.h"
+#include "copyright.cf"
 
 #define LOCKFILE_NAME	"/var/run/groupd.pid"
 #define CLIENT_NALLOC	32
@@ -62,6 +63,67 @@ static int do_write(int fd, void *buf, size_t count)
 	return 0;
 }
 
+void read_ccs_name(char *path, char *name)
+{
+	char *str;
+	int error;
+
+	error = ccs_get(ccs_handle, path, &str);
+	if (error || !str)
+		return;
+
+	strcpy(name, str);
+
+	free(str);
+}
+
+void read_ccs_yesno(char *path, int *yes, int *no)
+{
+	char *str;
+	int error;
+
+	*yes = 0;
+	*no = 0;
+
+	error = ccs_get(ccs_handle, path, &str);
+	if (error || !str)
+		return;
+
+	if (!strcmp(str, "yes"))
+		*yes = 1;
+
+	else if (!strcmp(str, "no"))
+		*no = 1;
+
+	free(str);
+}
+
+void read_ccs_int(char *path, int *config_val)
+{
+	char *str;
+	int val;
+	int error;
+
+	error = ccs_get(ccs_handle, path, &str);
+	if (error || !str)
+		return;
+
+	val = atoi(str);
+
+	if (val < 0) {
+		log_print("ignore invalid value %d for %s", val, path);
+		return;
+	}
+
+	*config_val = val;
+	log_debug("%s is %u", path, val);
+	free(str);
+}
+
+#define GROUPD_COMPAT_PATH "/cluster/group/@groupd_compat"
+#define GROUPD_WAIT_PATH "/cluster/group/@groupd_wait"
+#define GROUPD_MODE_DELAY_PATH "/cluster/group/@groupd_mode_delay"
+
 int setup_ccs(void)
 {
 	int i = 0, cd;
@@ -75,6 +137,18 @@ int setup_ccs(void)
 
 	ccs_handle = cd;
 
+	/* These config values are set from cluster.conf only if they haven't
+	   already been set on the command line. */
+
+	if (!optd_groupd_compat)
+		read_ccs_int(GROUPD_COMPAT_PATH, &cfgd_groupd_compat);
+
+	if (!optd_groupd_wait)
+		read_ccs_int(GROUPD_WAIT_PATH, &cfgd_groupd_wait);
+
+	if (!optd_groupd_mode_delay)
+		read_ccs_int(GROUPD_MODE_DELAY_PATH, &cfgd_groupd_mode_delay);
+
 	return 0;
 }
 
@@ -311,6 +385,7 @@ enum {
 	DO_GET_GROUP,
 	DO_DUMP,
 	DO_LOG,
+	DO_GET_VERSION,
 };
 
 int get_action(char *buf)
@@ -355,6 +430,9 @@ int get_action(char *buf)
 	if (!strncmp(act, "get_group", 16))
 		return DO_GET_GROUP;
 
+	if (!strncmp(act, "get_version", 16))
+		return DO_GET_VERSION;
+
 	if (!strncmp(act, "dump", 16))
 		return DO_DUMP;
 
@@ -544,6 +622,23 @@ static int do_get_group(int ci, int argc, char **argv)
 	return 0;
 }
 
+static int do_get_version(int ci)
+{
+	int mode;
+	int rv;
+
+	if (group_mode == GROUP_PENDING)
+		mode = -EAGAIN;
+	else
+		mode = group_mode;
+
+	rv = do_write(client[ci].fd, &mode, sizeof(mode));
+	if (rv < 0)
+		log_print("do_get_version write error");
+
+	return 0;
+}
+
 static int do_dump(int fd)
 {
 	int len;
@@ -657,6 +752,10 @@ static void process_connection(int ci)
 		do_get_group(ci, argc, argv);
 		break;
 
+	case DO_GET_VERSION:
+		do_get_version(ci);
+		break;
+
 	case DO_DUMP:
 		do_dump(client[ci].fd);
 		close(client[ci].fd);
@@ -728,6 +827,12 @@ void cluster_dead(int ci)
 	daemon_quit = 1;
 }
 
+#define min(x, y) ({                            \
+	typeof(x) _min1 = (x);                  \
+	typeof(y) _min2 = (y);                  \
+	(void) (&_min1 == &_min2);              \
+	_min1 < _min2 ? _min1 : _min2; })
+
 static void loop(void)
 {
 	int poll_timeout = -1;	
@@ -749,6 +854,15 @@ static void loop(void)
 	if (rv < 0)
 		goto out;
 
+	setup_logging();
+
+	if (cfgd_groupd_compat == 0)
+		group_mode = GROUP_LIBCPG;
+	else if (cfgd_groupd_compat == 1)
+		group_mode = GROUP_LIBGROUP;
+	else if (cfgd_groupd_compat == 2)
+		group_mode = GROUP_PENDING;
+
 	rv = check_uncontrolled_groups();
 	if (rv < 0)
 		goto out;
@@ -793,6 +907,13 @@ static void loop(void)
 			rv = 0;
 			rv += process_apps();
 		} while (rv);
+
+		poll_timeout = -1;
+
+		if (group_mode == GROUP_PENDING) {
+			group_mode_check_timeout();
+			poll_timeout = 1000 * min(cfgd_groupd_wait, cfgd_groupd_mode_delay);
+		}
 	}
  out:
 	close_ccs();
@@ -853,11 +974,24 @@ static void print_usage(void)
 	printf("Options:\n");
 	printf("\n");
 	printf("  -D	       Enable debugging code and don't fork\n");
+	printf("  -L <num>     Enable (1) or disable (0) debugging to logsys (default %d)\n", DEFAULT_DEBUG_LOGSYS);
+	printf("  -g <num>     group compatibility mode, 0 off, 1 on, 2 detect\n");
+	printf("               0: use libcpg, no backward compat, best performance\n");
+	printf("               1: use libgroup for compat with cluster2/rhel5\n");
+	printf("               2: detect old, or mode 1, nodes that require compat,\n"
+	       "               use libcpg if none found\n");
+	printf("               Default is %d\n", DEFAULT_GROUPD_COMPAT);
+	printf("  -w <secs>    seconds to wait for a node's version message before\n"
+	       "               assuming an old version requiring compat mode\n");
+	printf("               Default is %d\n", DEFAULT_GROUPD_WAIT);
+	printf("  -d <secs>    seconds to delay the mode selection to give time\n"
+	       "               for an old version to join and force compat mode\n");
+	printf("               Default is %d\n", DEFAULT_GROUPD_MODE_DELAY);
 	printf("  -h	       Print this help, then exit\n");
 	printf("  -V	       Print program version information, then exit\n");
 }
 
-#define OPTION_STRING "DhVv"
+#define OPTION_STRING "L:Dg:w:d:hVv"
 
 static void read_arguments(int argc, char **argv)
 {
@@ -873,6 +1007,26 @@ static void read_arguments(int argc, char **argv)
 			daemon_debug_opt = 1;
 			break;
 
+		case 'L':
+			optd_debug_logsys = 1;
+			cfgd_debug_logsys = atoi(optarg);
+			break;
+
+		case 'g':
+			optd_groupd_compat = 1;
+			cfgd_groupd_compat = atoi(optarg);
+			break;
+
+		case 'w':
+			optd_groupd_wait = 1;
+			cfgd_groupd_wait = atoi(optarg);
+			break;
+
+		case 'd':
+			optd_groupd_mode_delay = 1;
+			cfgd_groupd_mode_delay = atoi(optarg);
+			break;
+
 		case 'h':
 			print_usage();
 			exit(EXIT_SUCCESS);
@@ -885,7 +1039,7 @@ static void read_arguments(int argc, char **argv)
 		case 'V':
 			printf("groupd %s (built %s %s)\n",
 				RELEASE_VERSION, __DATE__, __TIME__);
-			/* printf("%s\n", REDHAT_COPYRIGHT); */
+			printf("%s\n", REDHAT_COPYRIGHT);
 			exit(EXIT_SUCCESS);
 			break;
 
@@ -905,6 +1059,11 @@ static void read_arguments(int argc, char **argv)
 			break;
 		};
 	}
+
+	if (!optd_debug_logsys && getenv("GROUPD_DEBUG")) {
+		optd_debug_logsys = 1;
+		cfgd_debug_logsys = atoi(getenv("GROUPD_DEBUG"));
+	}
 }
 
 static void set_oom_adj(int val)
@@ -946,6 +1105,8 @@ int main(int argc, char *argv[])
 	for (i = 0; i < MAX_LEVELS; i++)
 		INIT_LIST_HEAD(&gd_levels[i]);
 
+	init_logging();
+
 	read_arguments(argc, argv);
 
 	lockfile();
@@ -956,7 +1117,6 @@ int main(int argc, char *argv[])
 			exit(EXIT_FAILURE);
 		}
 	}
-	openlog("groupd", LOG_PID, LOG_DAEMON);
 	signal(SIGTERM, sigterm_handler);
 
 	set_scheduler();
@@ -996,4 +1156,15 @@ int dump_wrap;
 struct list_head gd_groups;
 struct list_head gd_levels[MAX_LEVELS];
 uint32_t gd_event_nr;
+int group_mode;
+
+int optd_groupd_compat;
+int optd_groupd_wait;
+int optd_groupd_mode_delay;
+int optd_debug_logsys;
+
+int cfgd_groupd_compat     = DEFAULT_GROUPD_COMPAT;
+int cfgd_groupd_wait       = DEFAULT_GROUPD_WAIT;
+int cfgd_groupd_mode_delay = DEFAULT_GROUPD_MODE_DELAY;
+int cfgd_debug_logsys      = DEFAULT_DEBUG_LOGSYS;
 
