@@ -1,18 +1,17 @@
 #include "gfs_daemon.h"
 #include "config.h"
 #include <pthread.h>
+#include "copyright.cf"
+
 #include <linux/netlink.h>
 
 #define LOCKFILE_NAME	"/var/run/gfs_controld.pid"
 #define CLIENT_NALLOC   32
-#define GROUP_LIBGROUP  2
-#define GROUP_LIBCPG    3
 
 static int client_maxi;
 static int client_size;
 static struct client *client;
 static struct pollfd *pollfd;
-static int group_mode;
 static pthread_t query_thread;
 static pthread_mutex_t query_mutex;
 
@@ -1003,9 +1002,11 @@ void cluster_dead(int ci)
 	daemon_quit = 1;
 }
 
-static void dead_dlmcontrol(int ci)
+static void dlmcontrol_dead(int ci)
 {
-	log_error("dlm_controld poll error %x", pollfd[ci].revents);
+	if (!list_empty(&mountgroups))
+		log_error("dlm_controld is gone");
+	client_dead(ci);
 }
 
 static void loop(void)
@@ -1036,6 +1037,8 @@ static void loop(void)
 	if (rv < 0)
 		goto out;
 
+	setup_logging();
+
 	rv = setup_uevent();
 	if (rv < 0)
 		goto out;
@@ -1050,13 +1053,10 @@ static void loop(void)
 		client_add(rv, process_groupd, cluster_dead);
 
 		group_mode = GROUP_LIBGROUP;
-
-		if (cfgd_groupd_compat == 2) {
-			/* set_group_mode(); */
-			/* might set group_mode to GROUP_LIBCPG */
-			group_mode = GROUP_LIBGROUP;
-		}
+		if (cfgd_groupd_compat == 2)
+			set_group_mode();
 	}
+	log_debug("group_mode %d compat %d", group_mode, cfgd_groupd_compat);
 
 	if (group_mode == GROUP_LIBCPG) {
 
@@ -1072,7 +1072,7 @@ static void loop(void)
 		rv = setup_dlmcontrol();
 		if (rv < 0)
 			goto out;
-		client_add(rv, process_dlmcontrol, dead_dlmcontrol);
+		client_add(rv, process_dlmcontrol, dlmcontrol_dead);
 
 	} else if (group_mode == GROUP_LIBGROUP) {
 
@@ -1158,6 +1158,7 @@ static void loop(void)
  out:
 	if (cfgd_groupd_compat)
 		close_groupd();
+	close_logging();
 	close_ccs();
 	close_cman();
 
@@ -1216,9 +1217,12 @@ static void print_usage(void)
 	printf("Options:\n");
 	printf("\n");
 	printf("  -D           Enable debugging code and don't fork\n");
-	printf("  -g <num>     groupd compatibility, 0 off, 1 on\n");
-	printf("               on: use libgroup, compat with cluster2/stable2/rhel5\n");
-	printf("               off: use libcpg, no backward compatability\n");
+	printf("  -L <num>     Enable (1) or disable (0) debugging to logsys (default %d)\n", DEFAULT_DEBUG_LOGSYS);
+	printf("  -g <num>     groupd compatibility mode, 0 off, 1 on, 2 detect\n");
+	printf("               0: use libcpg, no backward compat, best performance\n");
+	printf("               1: use libgroup for compat with cluster2/rhel5\n");
+	printf("               2: use groupd to detect old, or mode 1, nodes that\n"
+	       "               require compat, use libcpg if none found\n");
 	printf("               Default is %d\n", DEFAULT_GROUPD_COMPAT);
 	printf("  -w <num>     Enable (1) or disable (0) withdraw\n");
 	printf("               Default is %d\n", DEFAULT_ENABLE_WITHDRAW);
@@ -1240,7 +1244,7 @@ static void print_usage(void)
 	printf("  -V           Print program version information, then exit\n");
 }
 
-#define OPTION_STRING "DKg:w:f:q:d:p:Pl:o:t:c:a:hV"
+#define OPTION_STRING "L:DKg:w:f:q:d:p:Pl:o:t:c:a:hV"
 
 static void read_arguments(int argc, char **argv)
 {
@@ -1256,6 +1260,11 @@ static void read_arguments(int argc, char **argv)
 			daemon_debug_opt = 1;
 			break;
 
+		case 'L':
+			optd_debug_logsys = 1;
+			cfgd_debug_logsys = atoi(optarg);
+			break;
+
 		case 'g':
 			optd_groupd_compat = 1;
 			cfgd_groupd_compat = atoi(optarg);
@@ -1309,7 +1318,7 @@ static void read_arguments(int argc, char **argv)
 		case 'V':
 			printf("gfs_controld %s (built %s %s)\n",
 				RELEASE_VERSION, __DATE__, __TIME__);
-			/* printf("%s\n", REDHAT_COPYRIGHT); */
+			printf("%s\n", REDHAT_COPYRIGHT);
 			exit(EXIT_SUCCESS);
 			break;
 
@@ -1329,6 +1338,11 @@ static void read_arguments(int argc, char **argv)
 			break;
 		};
 	}
+
+	if (!optd_debug_logsys && getenv("GFS_CONTROLD_DEBUG")) {
+		optd_debug_logsys = 1;
+		cfgd_debug_logsys = atoi(getenv("GFS_CONTROLD_DEBUG"));
+	}
 }
 
 static void set_oom_adj(int val)
@@ -1365,6 +1379,8 @@ int main(int argc, char **argv)
 {
 	INIT_LIST_HEAD(&mountgroups);
 
+	init_logging();
+
 	read_arguments(argc, argv);
 
 	lockfile();
@@ -1375,7 +1391,6 @@ int main(int argc, char **argv)
 			exit(EXIT_FAILURE);
 		}
 	}
-	openlog("gfs_controld", LOG_PID, LOG_DAEMON);
 	signal(SIGTERM, sigterm_handler);
 
 	set_scheduler();
@@ -1421,4 +1436,5 @@ int plock_dump_len;
 int dmsetup_wait;
 cpg_handle_t libcpg_handle;
 int libcpg_flow_control_on;
+int group_mode;
 
