@@ -463,8 +463,6 @@ get_rg_state_local(char *name, rg_state_t *svcblk)
  *                      3 = DO NOT stop service, return RG_EFORWARD
  *			4 = DO NOT stop service, return RG_EAGAIN
  *			5 = DO NOT stop service, return RG_EFROZEN
- *			6 = DO NOT stop service, mark stopped and return
- *			    RG_SUCCESS (0)
  */
 int
 svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
@@ -491,7 +489,7 @@ svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
 	case RG_STATE_STARTING:
 	case RG_STATE_RECOVER:
 	case RG_STATE_MIGRATE:
-		if ((svcStatus->rs_owner != (uint32_t)my_id()) &&
+		if ((svcStatus->rs_owner != my_id()) &&
 		    memb_online(membership, svcStatus->rs_owner)) {
 			/*
 			   Service is running and the owner is online.
@@ -517,7 +515,7 @@ svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
 		}
 
 		if (svcStatus->rs_owner == 0 ||
-		    (svcStatus->rs_owner == (uint32_t)my_id())) {
+		    (svcStatus->rs_owner == my_id())) {
 			/*
 			   Service is marked as running locally or on
 			   0 (e.g. no member).  Safe
@@ -529,10 +527,9 @@ svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
 
 		/*
 		   Service is marked as running but node is down.
-		   Doesn't make much sense to stop it - but we need
-		   to mark it stopped
+		   Doesn't make much sense to stop it.
 		 */
-		ret = 6;
+		ret = 2;
 		break;
 
 	case RG_STATE_ERROR:
@@ -623,7 +620,7 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 	case RG_STATE_STARTED:
 	case RG_STATE_CHECK:
 	case RG_STATE_STARTING:
-		if (svcStatus->rs_owner == (uint32_t)my_id()) {
+		if (svcStatus->rs_owner == my_id()) {
 		    	/*
 			 * Service is already running locally
 			clulog(LOG_DEBUG,
@@ -633,7 +630,7 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 			break;
 		}
 
-		if (svcStatus->rs_owner != (uint32_t)my_id() &&
+		if (svcStatus->rs_owner != my_id() &&
 		    memb_online(membership, svcStatus->rs_owner)) {
 			/*
 			 * Service is running and the owner is online!
@@ -895,13 +892,13 @@ svc_migrate(char *svcName, int target)
 		return RG_EFAIL;
 	}
 
-	if (svcStatus.rs_owner == (uint32_t)target) {
+	if (svcStatus.rs_owner == target) {
 		rg_unlock(&lockp);
 		/* Do not allow migration to its current owner! */
 		return 0;
 	}
 
-	if (svcStatus.rs_owner != (uint32_t)my_id()) {
+	if (svcStatus.rs_owner != my_id()) {
 		rg_unlock(&lockp);
 		return RG_EFORWARD;
 	}
@@ -978,8 +975,8 @@ svc_migrate(char *svcName, int target)
 		return ret;
 	}
 
-	if (svcStatus.rs_last_owner != (uint32_t)my_id() ||
-	    svcStatus.rs_owner != (uint32_t)target ||
+	if (svcStatus.rs_last_owner != my_id() ||
+	    svcStatus.rs_owner != target ||
 	    svcStatus.rs_state != RG_STATE_MIGRATE) {
 		rg_unlock(&lockp);
 		return ret;
@@ -1151,7 +1148,7 @@ svc_status(char *svcName)
 		/* Don't check status if the service is frozen */
 		return 0;
 
-	if (svcStatus.rs_owner != (uint32_t)my_id())
+	if (svcStatus.rs_owner != my_id())
 		/* Don't check status for anything not owned */
 		return 0;
 
@@ -1171,8 +1168,7 @@ svc_status(char *svcName)
 
 
 static inline int
-handle_started_status(char *svcName, int ret,
-		      rg_state_t __attribute__((unused)) *svcStatus)
+handle_started_status(char *svcName, int ret, rg_state_t *svcStatus)
 {
 	int newowner;
 
@@ -1302,16 +1298,6 @@ _svc_stop(char *svcName, int req, int recover, uint32_t newstate)
 		clulog(LOG_DEBUG, "Unable to stop RG %s in %s state\n",
 		       svcName, rg_state_str(svcStatus.rs_state));
 		return RG_EFAIL;
-	case 6:
-		/* Mark stopped, but do not do anything */
-		svcStatus.rs_last_owner = svcStatus.rs_owner;
-		svcStatus.rs_owner = 0;
-		svcStatus.rs_state = RG_STATE_STOPPED;
-		if (set_rg_state(svcName, &svcStatus) != 0) {
-			rg_unlock(&lockp);
-			return RG_EFAIL;
-		}
-		/* FALLTHROUGH */
 	case 2:
 		rg_unlock(&lockp);
 		return RG_ESUCCESS;
@@ -1472,7 +1458,7 @@ svc_fail(char *svcName)
 	}
 
 	if ((svcStatus.rs_state == RG_STATE_STARTED) &&
-	    (svcStatus.rs_owner != (uint32_t)my_id())) {
+	    (svcStatus.rs_owner != my_id())) {
 		rg_unlock(&lockp);
 		clulog(LOG_DEBUG, "Unable to disable RG %s in %s state\n",
 		       svcName, rg_state_str(svcStatus.rs_state));
@@ -1520,8 +1506,7 @@ _svc_freeze(char *svcName, int enabled)
 		return RG_EFAIL;
 	}
 
-	clulog(LOG_DEBUG, "Handling %s request for RG %s\n", enabled?"freeze":"unfreeze",
-	       svcName);
+	clulog(LOG_DEBUG, "Handling %s request for RG %s\n", svcName, enabled?"freeze":"unfreeze");
 
 	if (get_rg_state(svcName, &svcStatus) != 0) {
 		rg_unlock(&lockp);
@@ -1552,7 +1537,7 @@ _svc_freeze(char *svcName, int enabled)
 
 	default:
 		rg_unlock(&lockp);
-		return RG_EAGAIN;
+		return RG_EFAIL;
 		break;
 	}
 
@@ -1610,7 +1595,7 @@ svc_start_remote(char *svcName, int request, uint32_t target)
 
 	/* Send relocate message to the other node */
 	if (msg_send(&ctx, &msg_relo, sizeof (SmMessageSt)) < 
-	    (int)sizeof (SmMessageSt)) {
+	    sizeof (SmMessageSt)) {
 		clulog(LOG_ERR,
 		       "#59: Error sending remote-start request to member #%d\n",
 		       target);
@@ -2057,8 +2042,8 @@ handle_start_remote_req(char *svcName, int req)
 
 	/* XXX ok, so we need to say "should I start this if I was the
 	   only cluster member online */
-	for (x = 0; x < (int)membership->cml_count; x++) {
-		if (membership->cml_members[x].cn_nodeid == (int)me)
+	for (x = 0; x < membership->cml_count; x++) {
+		if (membership->cml_members[x].cn_nodeid == me)
 			continue;
 
 		membership->cml_members[x].cn_member = 0;
